@@ -44,6 +44,8 @@ struct md_acme_acct_t {
     const char *tos_required;       /* terms of service asked for by CA */
     const char *agreement;          /* terms of service agreed to by user */
     const char *orders;             /* URL where certificate orders are found (ACMEv2) */
+    const char *eab_kid;            /* external account binding keyid used or NULL */
+    const char *eab_hmac;           /* external account binding hmac used or NULL */
     struct md_json_t *registration; /* data from server registration */
 };
 
@@ -104,21 +106,20 @@ const char *md_acme_get_agreement(md_acme_t *acme);
  * Find an existing account in the local store. On APR_SUCCESS, the acme
  * instance will have a current, validated account to use.
  */ 
-apr_status_t md_acme_find_acct(md_acme_t *acme, md_store_t *store);
+apr_status_t md_acme_find_acct_for_md(md_acme_t *acme, md_store_t *store, const md_t *md);
 
 /**
- * Find the account id for a given account url. 
+ * Find the account id for a given md.
  */
-apr_status_t md_acme_acct_id_for_url(const char **pid, md_store_t *store, 
-                                     md_store_group_t group, const char *url, apr_pool_t *p);
+apr_status_t md_acme_acct_id_for_md(const char **pid, md_store_t *store,
+                                    md_store_group_t group, const md_t *md, apr_pool_t *p);
 
 /**
- * Create a new account at the ACME server. The
+ * Create a new account at the ACME server for an MD. The
  * new account is the one used by the acme instance afterwards, on success.
  */
 apr_status_t md_acme_acct_register(md_acme_t *acme, md_store_t *store, 
-                                   apr_pool_t *p, apr_array_header_t *contacts, 
-                                   const char *agreement);
+                                   const md_t *md, apr_pool_t *p);
 
 apr_status_t md_acme_acct_save(md_store_t *store, apr_pool_t *p, md_acme_t *acme,  
                                const char **pid, struct md_acme_acct_t *acct, 
@@ -133,4 +134,15 @@ apr_status_t md_acme_acct_load(struct md_acme_acct_t **pacct, struct md_pkey_t *
                                md_store_t *store, md_store_group_t group, 
                                const char *name, apr_pool_t *p);
 
+/*
+ * Return != 0 iff the account can be used for the ACME url.
+ */
+int md_acme_acct_matches_url(md_acme_acct_t *acct, const char *url);
+
+/*
+ * Return != 0 iff the account can be used for the MD, including
+ * its CA url and EAB settings.
+ */
+int md_acme_acct_matches_md(md_acme_acct_t *acct, const md_t *md);
+
 #endif /* md_acme_acct_h */
