@@ -20,6 +20,16 @@
 #include <stdio.h>
 #include <apr_file_io.h>
 
+#include <apr_version.h>
+
+#if APR_VERSION_AT_LEAST(1,6,0)
+#include <apr_cstr.h>
+#else
+/* Define a (poor) backwards-compatibility entry point for
+ * apr_cstr_casecmp() from APR 1.6. */
+#define apr_cstr_casecmp apr_natcasecmp
+#endif
+
 struct apr_array_header_t;
 struct apr_table_t;
 
@@ -133,7 +143,7 @@ int md_array_str_add_missing(struct apr_array_header_t *dest,
 /* process execution */
 
 apr_status_t md_util_exec(apr_pool_t *p, const char *cmd, const char * const *argv,
-                          struct apr_array_header_t *env, int *exit_code);
+                          int *exit_code);
 
 /**************************************************************************************************/
 /* dns name check */
@@ -174,6 +184,11 @@ struct apr_array_header_t *md_dns_make_minimal(apr_pool_t *p,
  */
 int md_dns_domains_match(const apr_array_header_t *domains, const char *name);
 
+/**
+ * @return != 0 iff `name` is matched by a wildcard pattern in `domains`
+ */
+int md_is_wild_match(const apr_array_header_t *domains, const char *name);
+
 /**************************************************************************************************/
 /* file system related */
 
@@ -189,6 +204,7 @@ apr_status_t md_util_path_merge(const char **ppath, apr_pool_t *p, ...);
 
 apr_status_t md_util_is_dir(const char *path, apr_pool_t *pool);
 apr_status_t md_util_is_file(const char *path, apr_pool_t *pool);
+apr_status_t md_util_is_unix_socket(const char *path, apr_pool_t *pool);
 int md_file_exists(const char *fname, apr_pool_t *p);
 
 typedef apr_status_t md_util_file_cb(void *baton, struct apr_file_t *f, apr_pool_t *p);
