@@ -1,9 +1,10 @@
 import pytest
 import os
 
-from .env import H2Conf
+from .env import H2Conf, H2TestEnv
 
 
+@pytest.mark.skipif(condition=H2TestEnv.is_unsupported, reason="mod_http2 not supported here")
 class TestLoadCgi:
 
     @pytest.fixture(autouse=True, scope='class')
@@ -24,7 +25,8 @@ class TestLoadCgi:
         assert 0 == r.results["h2load"]["status"]["5xx"]
     
     # test POST on cgi, where input is read
-    def test_h2_711_10(self, env):
+    def test_h2_711_10(self, env, repeat):
+        assert env.is_live()
         url = env.mkurl("https", "test1", "/echo.py")
         n = 100
         m = 5
@@ -39,7 +41,8 @@ class TestLoadCgi:
         self.check_h2load_ok(env, r, n)
 
     # test POST on cgi via http/1.1 proxy, where input is read
-    def test_h2_711_11(self, env):
+    def test_h2_711_11(self, env, repeat):
+        assert env.is_live()
         url = env.mkurl("https", "test1", "/proxy/echo.py")
         n = 100
         m = 5
@@ -54,7 +57,8 @@ class TestLoadCgi:
         self.check_h2load_ok(env, r, n)
 
     # test POST on cgi via h2proxy, where input is read
-    def test_h2_711_12(self, env):
+    def test_h2_711_12(self, env, repeat):
+        assert env.is_live()
         url = env.mkurl("https", "test1", "/h2proxy/echo.py")
         n = 100
         m = 5
